# -*- coding: utf-8 -*-

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

"""
CLI to manage ccsdb
"""

from __future__ import print_function

import argparse
import json
import logging

import ccsdb

_log = logging.getLogger(__name__)


def setup_parser():
    '''
    Set the main arguments.
    '''
    parser = argparse.ArgumentParser(
        prog="ccsdb-cli",
        version=ccsdb.__version__
    )
    # General connection options
    parser.add_argument(
        '--debug', action='store_true',
        help="Outputs bunches of debugging info")

    subparsers = parser.add_subparsers(title='actions')

    # INIT_DB
    parser_init_db = subparsers.add_parser(
        'init_db', help='Initialize the database')
    parser_init_db.set_defaults(func=init_db)
    parser_init_db.add_argument(
        '--destructive', action="store_true", default=False,
        help="Drop all the tables before re-creating them")

    # DEV_DATA
    parser_dev_data = subparsers.add_parser(
        'dev-data', help='Insert some dev data to the database')
    parser_dev_data.set_defaults(func=dev_data)

    return parser


def init_db(args):
    """ Initialize the database. """
    from ccsdb.app import _db
    _log.info('Initializing the database')
    if _log.getEffectiveLevel() <= 10:
        _db.engine.echo = True

    if args.destructive:
        _log.info(' - Dropping all tables first')
        _db.drop_all()
    _db.create_all()
    _log.info('Done')


def dev_data(args):
    """ Insert some dev data in the database. """
    data = [
        {
            "commit": "000a4a4bfd777dedae2599ec503af2dc4b6ec70d",
            "date_created": "2017-06-05T21:12:53.519740",
            "date_updated": "2017-06-05T21:12:53.519747",
            "namespace": "rpms",
            "branch": "rawhide",
            "project": "perl-XML-Generator",
            "status": "ignored"
        },
        {
            "commit": "00101ebba9a10a688b5508e60a311f7eb606ec23",
            "date_created": "2017-06-05T22:31:48.783426",
            "date_updated": "2017-06-05T22:31:48.783432",
            "namespace": "rpms",
            "branch": "rawhide",
            "project": "inn",
            "status": "queued"
        },
        {
            "commit": "00101ebba9a10a688b5508e60a311f7eb606ec23",
            "date_created": "2017-06-05T22:31:48.783426",
            "date_updated": "2017-06-05T22:31:48.783432",
            "namespace": "rpms",
            "branch": "f25",
            "project": "inn",
            "status": "failed"
        },
        {
            "commit": "00103a4ce578b6c12d674d4099db7edf6dad2588",
            "date_created": "2017-06-04T07:53:46.987810",
            "date_updated": "2017-06-04T07:53:46.987817",
            "namespace": "rpms",
            "branch": "rawhide",
            "project": "perl-Canary-Stability",
            "status": "running"
        }
    ]

    from ccsdb.app import _db, Branch, Commit, Message
    _log.info('Adding some data to the database')

    commits = set()

    for entry in data:
        commit = entry['commit']

        # Create the commits
        if commit not in commits:
            db_commit = Commit(
                commit_hash=entry['commit'],
                project=entry['project'],
                namespace=entry['namespace'],
            )
            _db.session.add(db_commit)
            _db.session.flush()
            commits.add(commit)

        db_branch = Branch(
            commit_hash=commit,
            branch=entry['branch'],
            status=entry['status'],
        )
        _db.session.add(db_branch)
        _db.session.flush()

        # Record the message in the DB
        message = Message(
            branch_id=db_branch.id,
            commit_hash=commit,
            message=json.dumps(entry)
        )
        _db.session.add(message)

    _db.session.commit()
    _log.info('Done')


def main():
    ''' Main function '''
    # Set up parser for global args
    parser = setup_parser()
    # Parse the commandline
    try:
        args = parser.parse_args()
    except argparse.ArgumentTypeError as err:
        print("\nError: {0}".format(err))
        return 3

    logging.basicConfig()
    _log.setLevel(logging.INFO)
    if args.debug:
        _log.setLevel(logging.DEBUG)

    return_code = 0
    try:
        args.func(args)
    except KeyboardInterrupt:
        print("\nInterrupted by user.")
        return_code = 1
    except argparse.ArgumentError as err:
        _log.debug('ArgparseError')
        print('{0}'.format(err.message))
        return_code = 4
    except Exception as err:
        print('Error: {0}'.format(err))
        _log.exception("Generic error catched:")
        return_code = 2
    return return_code


if __name__ == '__main__':  # pragma: no cover
    main()
